/*
 * I18N core class stuffs
 */

/*
 * common
 */
typedef struct _i18n_core_class	*_i18n_class_t;
typedef struct _i18n_core	*_i18n_object_t;

#if _I18N_USE_GCC
#define _I18nFillField(m,v) .m=v
#else
#define _I18nFillField(m,v) v
#endif


int _i18n_iskindof(_i18n_class_t c, char **cookie);
#define _I18nCheckKindOf(c,n) _I18N_ASSERT(_i18n_iskindof(c,#n))
#define _I18nCheckObj(i)   ((void)&(i)->core)
#define _I18nCheckClass(c) ((void)&(c)->ccore)
#define _I18nClassOf(i) ((i)->core.klass)
#define _I18nOToClass(c) (_I18nCheckObj(c),(_i18n_class_t)(c))
#define _I18nCToClass(c) (_I18nCheckClass(c),(_i18n_class_t)(c))
#define _I18nOToObject(i) (_I18nCheckObj(i),(_i18n_object_t)(i))
#define _I18nOToNamedObject(i,n)\
	(_I18nCheckObj(i),\
	 _I18nCheckKindOf(_I18nClassOf(i),_I18nGetCookie(n)),\
	 (struct n*)i)
#define _I18nCToNamedClass(c,n) \
	(_I18nCheckClass(c),_I18nCheckKindOf(c,_I18nGetCookie(n)),\
	 (struct n##_class*)c)

#define _I18nRef(i) \
	((void)((i)->core.refCount>=0 && \
		(_I18N_DPR(("Ref(%s:%s):%d->%d\n", \
		            _I18nClassOf(i)->ccore.cookie, #i, \
		            (i)->core.refCount, (i)->core.refCount+1)),1) && \
                (i)->core.refCount++))
#define _I18nUnref(i)	\
	((void)((i)->core.refCount>0 && \
		(_I18N_DPR(("Unref(%s:%s):%d->%d\n", \
		            _I18nClassOf(i)->ccore.cookie, #i, \
		            (i)->core.refCount, (i)->core.refCount-1)),1) && \
	        --(i)->core.refCount<=0 && \
	        _I18nClassOf(i)->ccore.release && \
	        (_I18nClassOf(i)->ccore.release(_I18nOToCore(i)),0)))

#define _I18n_T_FuncDefField(n,r,m,a)   r (*m) a;
#define _I18n_T_FuncFillField(n,r,m,a)  _I18nFillField(m,_D_##n##_##m),
#define _I18n_T_FuncDeclExtern(n,r,m,a) extern r n##_##m a;
#define _I18n_T_VarDefField(n,t,m)      t m;
#define _I18n_T_VarFillField(n,t,m)     _I18nFillField(m,_D_##n##_##m),
#define _I18n_T_VarDeclExtern(n,t,m)

#define _I18n_T_DeclAllFieldsExtern(n,f) \
	f(n,_I18n_T_FuncDeclExtern, _I18n_T_VarDeclExtern)
#define _I18n_T_DefAllFields(n,f) \
	f(n,_I18n_T_FuncDefField,_I18n_T_VarDefField)
#define _I18n_T_FillAllFields(n,f) \
	f(n,_I18n_T_FuncFillField,_I18n_T_VarFillField)

#define _I18n_T_DefPart(c,p) \
	struct c##_part { _I18n_T_DefAllFields(p,_I18n_T_Fields_##c##_part) }
#define _I18n_T_FillPart(c,p) \
	_I18nFillField(p,{_I18n_T_FillAllFields(p,_I18n_T_Fields_##c##_part)}),

#define _I18n_T_DefPartOfRec(c,p)	struct c##_part p;
#define _I18n_T_FillPartOfRec(c,p)	_I18n_T_FillPart(c,p)

#define _I18n_T_DefRec(c)  struct c { _I18n_T_Rec_##c(_I18n_T_DefPartOfRec) }
#define _I18n_T_DeclRec(c) extern struct c _classRec_##c;
#define _I18n_T_FillRec(c) \
struct c _classRec_##c = { _I18n_T_Rec_##c(_I18n_T_FillPartOfRec) }
#define _I18nGetClass(c) ((_i18n_class_t)&_classRec_##c)

#define _I18n_T_DeclCookie(c) char *_cookie_##c
#define _I18n_T_FillCookie(c) char *_cookie_##c = #c
#define _I18nGetCookie(c) (&_cookie_##c)


/*
 * core class / core class object
 */

/* cast methods with static type-checking */

/* any object to core object */
#define _I18nOToCore(i)      ((void)_I18nClassOf(i),(struct _i18n_core *)(i))
/* any class object to core class object */
#define _I18nCToCoreClass(c) ((void)(c)->ccore,(struct _i18n_core_class *)(c))


/* templates */
#define _I18n_T_Fields__i18n_core_part(n,f,v) \
	v(n,_i18n_class_t,	klass) \
	v(n,int,		refCount) \
	v(n,void *,		dynamic) \

#define _I18n_T_Fields__i18n_core_class_part(n,f,v) \
	v(n,char **,		cookie) \
	v(n,_i18n_class_t,	super) \
	f(n,void,		release,	(_i18n_object_t)) \

#define _I18n_T_Rec__i18n_core(f) \
	f(_i18n_core,core)

#define _I18n_T_Rec__i18n_core_class(f) \
	_I18n_T_Rec__i18n_core(f) \
	f(_i18n_core_class,ccore)


_I18n_T_DefPart(_i18n_core,core);
_I18n_T_DefRec(_i18n_core);
_I18n_T_DeclCookie(_i18n_core);

_I18n_T_DefPart(_i18n_core_class,ccore);
_I18n_T_DefRec(_i18n_core_class);

_I18n_T_DeclRec(_i18n_core_class);

/* end of file */
