#include <stdlib.h>
#include <dlfcn.h>
#include <i18n/i18n_types.h>
#include <i18n/i18n_debug.h>
#include <i18n/i18n_class.h>
#include <i18n/i18n_static.h>
#include <i18n/i18n_dynamic.h>

_i18n_object_t
_i18n_load_object(char *file, void *arg)
{
	void *h, *f;
	_i18n_object_t ret;

	if (!(h=dlopen(file, RTLD_NOW))) {
		_I18N_DPR(("dlopen(%s) failed: %s.\n", file, dlerror()));
		return NULL;
	}
	if (!(f=dlsym(h, _I18N_DYNAMIC_GET_SYM))) {
		_I18N_DPR(("dlsym(%s) failed: %s.\n",
		    _I18N_DYNAMIC_GET_SYM, dlerror()));
		return NULL;
	}

	ret = (*_I18N_DYNAMIC_GET_CAST(f))(arg);
	if (ret) {
		_I18nRef(ret);
		ret->core.dynamic = h;
	}

	return ret;
}

void
_i18n_unload_object(_i18n_object_t o)
{
	void *f;

	if ((f=dlsym(o->core.dynamic, _I18N_DYNAMIC_UNINIT_SYM))) {
		(*_I18N_DYNAMIC_UNINIT_CAST(f))(o);
	} else
		_I18N_DPR(("dlsym(%s) failed: %s.\n",
		    _I18N_DYNAMIC_GET_SYM, dlerror()));

	dlclose(o->core.dynamic);
}

#define _D_core_klass		_I18nGetClass(_i18n_static_class)
#define _D_core_refCount	0
#define _D_core_dynamic		NULL
#define _D_ccore_super		NULL
#define _D_ccore_cookie		_I18nGetCookie(_i18n_core)
#define _D_ccore_release	&_i18n_unload_object
_I18n_T_FillRec(_i18n_dynamic_class);

/* end of file */
